 # -*- coding: ISO-8859-1 -*-
""" capellaScript -- Copyright (c) 2019 Peter Becker
>>> Mididrums
    Erstellt eine oder mehrere Schlagzeugstimmen
    aus einer importierten Midi Datei
    ||
    Bei Problemen bitte eine Mail an peter_becker@freenet.de
<<<
<?xml version="1.0" encoding="ISO-8859-1"?>
<info>
  <lang id="en">
    <title>Mididrums</title>
    <descr>
      <p>Create Drum Notes from an imported Midi file

In case of problems please mail to peter_becker@freenet.de</p>
    </descr>
  </lang>

  <lang id="de">
    <title>Mididrums</title>
    <descr>
      <p>Erstellt eine oder mehrere Schlagzeugstimmen
    aus einer importierten Midi Datei

Bei Problemen bitte eine Mail an peter_becker@freenet.de</p>
    </descr>
  </lang>

  <lang id="nl">
    <title>Mididrums</title>
    <descr>
      <p>Genereert n of meer slagwerkstemmen uit
een gemporteerd Midi-bestand

Bij problemen s.v.p. een mail naar peter_becker@freenet.de</p>
    </descr>
  </lang>
</info>

History:    v 1.0.0 28.11.19 - PB Erste Ausgabe
            v 1.0.1 22.01.20 - WW Internationalisierung (en-de-nl)
            v 1.0.2 22.01.20 - PB Menue vereinfacht
            v 1.0.3 26.01.20 - WW Synonyme fr zu lange Worte (nl)

"""
english = {
    'notUsed'       :'Not Used',
    'up'            :'Up',
    'down'          :'Down',
    'normal'        :'Normal',
    'cross'         :'Cross',
    'triangle'      :'Triangle',
    'diamond'       :'Diamond',
    'square'        :'Square',
    'crossRing'     :'Cross with Circle',
    'percussion'    :'Percussion ',
    'image'         :'   Representation ',
    'head'          :'   Head ',
    'stemDir'       :'   Stem Direction ',
    'voiceName'     :'Voice Name',
    'drumStaffName' :'Name of Drum Staff',
    'selInstr'      :'Selection of Instruments'
}
german = {
    'notUsed'       :'nicht belegt',
    'up'            :'nach oben',
    'down'          :'nach unten',
    'normal'        :'normal',
    'cross'         :'Kreuz',
    'triangle'      :'Triangel',
    'diamond'       :'Rombus',
    'square'        :'Quadrat',
    'crossRing'     :'Kreuz mit Ring',
    'percussion'    :'Percussion ',
    'image'         :'   Darstellung ',
    'head'          :'   Kopfform ',
    'stemDir'       :'   Halsrichtung ',
    'voiceName'     :'Stimmenname',
    'drumStaffName' :'Name der Schlagzeugzeile',
    'selInstr'      :'Instrumenten Auswahl'
}
dutch = {
    'notUsed'       :'Niet gebruikt',
    'up'            :'Omhoog',
    'down'          :'Omlaag',
    'normal'        :'Normaal',
    'cross'         :'Kruis',
    'triangle'      :'Triangel',
    'diamond'       :'Rhombus',
    'square'        :'Vierkant',
    'crossRing'     :'Kruis met cirkel',
    'percussion'    :'Percussie ',
    'image'         :'   Weergave ',
    'head'          :'   Kopvorm ',
    'stemDir'       :'   Stokrichting ',
    'voiceName'     :'Naam notenbalk',
    'drumStaffName' :'Naam slagwerkbalk',
    'selInstr'      :'Selectie van instrumenten'
}

try:
    setStringTable(
        ("en", english),
        ("de", german),
        ("nl", dutch))
except:
    def tr(s):
        return german[s]

#------------------------------------------
#from xml.dom.minidom import NodeList, Node, Element
import xml.dom, tempfile
from caplib.capDOM import *
from xml.dom.minidom import NodeList, Node, Element

version = '1.0.3'

def addElementNode(el,tagName):
    # Neue Node zu "el" hinzufgen wenn Node "tagName" nicht existiert
    # ansonsten existierende Node zurckmelden
    global doc
    childs = el.childNodes
    for n in range(childs.length):
        if childs[n].nodeType ==childs[n].ELEMENT_NODE and childs[n].tagName == tagName:
            return childs[n]
    newChild = doc.createElement(tagName)
    el.appendChild(newChild)
    return newChild


def addNewElementNode(el,tagName):
    # add new Node with tagName "tagName" to el 
    global doc
    newChild = doc.createElement(tagName)
    el.appendChild(newChild)
    return newChild


def cleanStaff(note):
    # neue Schlagzeugzeile vorbereiten
    if note.tagName == 'clefSign':
        note.setAttribute('clef','P3')                                          # Schlagzeugschlssel setzen
    elif note.tagName == 'keySign':
        note.setAttribute('fifths','0')                                         # Tonart weg
    elif note.tagName == 'chord' or note.tagName == 'rest' or note.tagName == 'barline' or note.tagName == 'timeSign':
        note.parentNode.removeChild(note)                                       # und alle Noten, Pausen und Barlines lschen
        
        
def insertVoices(staff,voice):                          
    # neue Schlagzeugstimme in die neue Notenzeile einfgen
    clone = voice.cloneNode(True)
    voice.parentNode.insertBefore(clone, voice) 
    addElementNode(voice,'noteObjects')
    
    
def copyTime(obj,staffTarget):
    # Takt einfgen
    ti = obj.getAttribute('time')
    voices = staffTarget.getElementsByTagName('voices')
    for vs in range(voices.length):
        voice = voices[vs].getElementsByTagName('voice')
        for vo in range(voice.length):            
            noObjs = voice[vo].getElementsByTagName('noteObjects')
            for no in range(noObjs.length):       
                ts = addNewElementNode(noObjs[no],'timeSign')
                ts.setAttribute('time',ti)

def copyBarLine(obj,staffTarget):
    # festen Taktstrich einfgen
    ty = obj.getAttribute('type')
    voices = staffTarget.getElementsByTagName('voices')
    for vs in range(voices.length):
        voice = voices[vs].getElementsByTagName('voice')
        for vo in range(voice.length):            
            noObjs = voice[vo].getElementsByTagName('noteObjects')
            for no in range(noObjs.length):       
                #messageBox('noObjs='+ str(noObjs[no]) + '\nvoice=' + 'vo',str(voice[vo]) + '\n' + str(voice.length) + '\n' + str(vo))    
                ts = addNewElementNode(noObjs[no],'barline')
                ts.setAttribute('type',ty)    

def copyChord(obj,staffTarget,trans):
    # Note einfgen
    global inst1, inst2, inst3, inst4, inst5, dar1, dar2, dar3, dar4, dar5, kopf1, kopf2, kopf3, kopf4, kopf5, hals1, hals2, hals3, hals4, hals5
    
    instList     = [inst1,inst2,inst3,inst4,inst5]
    darList      = [dar1,dar2,dar3,dar4,dar5]
    kopfList     = [kopf1,kopf2,kopf3,kopf4,kopf5]
    halsList     = [hals1,hals2,hals3,hals4,hals5]
    MidiInst     = ['XXX','B20',
                    'C30','C31','D30','D31','E30','F30','F31','G30','G31','A30','A31','B30','C40','C41','D40','D41','E40','F40','F41','G40','G41','A40','A41','B40',
                    'C50','C51','D50','D51','E50','F50','F51','G50','G51','A50','A51','B50','C60','C61','D60','D61','E60','F60','F61','G60','G61','A60','A61','B60',
                    'C70','C71','D70','D71','E70','F70','F71','G70','G71','A70','A71','B70','C80','C81','D80','D81','E80','F80','F81','G80','G81','A80','A81','B80',
                    'C90','C91','D90','D91','E90','F90','F91','G90','G91','A90','A91','B90']
    kopfform     = ['normal','crossCircle','triangle','diamond','square','crossCircle2']
    halsrichtung = ['up','down']
    darstellung  = ['C5','D5','E5','F5','G5','A5','B5','C6','D6','E6','F6','G6','A6','B6','C7','D7','E7','F7','G7','A7','B7']
    
    duration = obj.getElementsByTagName('duration')                             # Infos des bergebenen chord extrahieren
    ba     = duration[0].getAttribute('base')
    dot = 0
    if duration[0].hasAttribute('dots'):
        dot = duration[0].getAttribute('dots')
    heads  = obj.getElementsByTagName('heads')
    headL  = heads[0].getElementsByTagName('head')
    step = '0'
    pcList = ['R','R','R','R','R']
    lenList = ['-1.5','-1','-0.5','1','1.5']
    
    for instX in range(5):                                                      # jetzt alle 5 Stimmen bearbeiten  
        voices = staffTarget.getElementsByTagName('voices') 
        voice  = voices[0].getElementsByTagName('voice')
        noObjs = voice[instX].getElementsByTagName('noteObjects')               # Anker fr noteObject der jeweiligen Target Stimme aufsetzen
        
        for hd in range(headL.length):                                          # Kpfe des bergebenen chords gegen die eingestellten Instrumente abgleichen
            if instList[instX] <> 0:                                            # nur wenn ein Instrument gesetzt ist berprfen  
                pitch = headL[hd].getAttribute('pitch')
                alter = headL[hd].getElementsByTagName('alter')
                if alter:
                    step = alter[0].getAttribute('step')
                else:
                    step = '0' 
                if step == '1':
                    pitch = pitch+'1'
                else:
                    pitch = pitch+'0'        
 
                if trans == '-24':
                    instIX = instList[instX] + 24
                else:         
                    instIX = instList[instX]
                GM = MidiInst[instIX]  
                
                if pitch == GM:                                                 # chord mit definiertem Instrument vergleichen 
                    pcList[instX] = 'C'                                         # Chord fr Stimme markiere
                    ch = addNewElementNode(noObjs[0],'chord')                   # und definierte Note einsetzen
                    du = addNewElementNode(ch,'duration')
                    di = addNewElementNode(ch,'display')
                    du.setAttribute('base',ba) 
                    if dot <> 0:
                         du.setAttribute('dots',dot) 
                    di.setAttribute('noAutoXShift','true')
                    st = addNewElementNode(ch,'stem')
                    st.setAttribute('dir',halsrichtung[halsList[instX]]) 
                    if halsrichtung[halsList[instX]] == 'up':
                         st.setAttribute('lengthening',lenList[instX]) 
                    hds = addNewElementNode(ch,'heads')
                    hd  = addNewElementNode(hds,'head')
                    hd.setAttribute('pitch',darstellung[darList[instX]]) 
                    if kopfList[instX] <> 0:
                        hd.setAttribute('shape',kopfform[kopfList[instX]])
                else:
                    if pcList[instX] == 'R':                                    # Falls Stimme noch auf Pause steht, unsichtbare Pause markieren
                        pcList[instX] = 'I'
            else:
                pcList[instX] = 'I'                                             # Falls Stimme nicht zugeordnet auf unsichtbare Pause setzen
                
    for instX in range(5):                                                      # jetzt Pausen in den 5 Stimmen setzen  
        voices = staffTarget.getElementsByTagName('voices') 
        voice  = voices[0].getElementsByTagName('voice')
        noObjs = voice[instX].getElementsByTagName('noteObjects')               # Anker fr noteObject der jeweiligen Target Stimme aufsetzen
        
        if pcList[instX] == 'R' or pcList[instX] == 'I':                        # Falls Pause oder unsichtbare Pause, Pause einfgen
            re = addNewElementNode(noObjs[0],'rest')
            du = addNewElementNode(re,'duration')
            di = addNewElementNode(re,'display')
            du.setAttribute('base',ba)
            di.setAttribute('noAutoXShift','true')
            di.setAttribute('churchStyle','true')
            if dot <> 0:
                du.setAttribute('dots',dot)             
        if pcList[instX] == 'I':                                                # Falls unsichtbare Pause, als unsichtbar markieren
            #di = addNewElementNode(re,'display')
            di.setAttribute('invisible','true')             
    
def copyRest(obj,staffTarget):
    # Pause einfgen
    duration = obj.getElementsByTagName('duration')
    ba = duration[0].getAttribute('base')
    dot = 0
    if duration[0].hasAttribute('dots'):
        dot = duration[0].getAttribute('dots')    
    voices = staffTarget.getElementsByTagName('voices')
    for vs in range(voices.length):
        voice = voices[vs].getElementsByTagName('voice')
        for vo in range(voice.length):            
            noObjs = voice[vo].getElementsByTagName('noteObjects')
            for no in range(noObjs.length):       
                #messageBox('noObjs='+ str(noObjs[no]) + '\nvoice=' + 'vo',str(voice[vo]) + '\n' + str(voice.length) + '\n' + str(vo)) 
                re = addNewElementNode(noObjs[no],'rest')
                du = addNewElementNode(re,'duration')
                di = addNewElementNode(re,'display')
                di.setAttribute('noAutoXShift','true')
                du.setAttribute('base',ba) 
                di.setAttribute('churchStyle','true')
    
    
def getElementObjects(objList):  # returns a List
    newList = NodeList()
    for n in range(objList.length):
        if objList[n].nodeType == objList[n].ELEMENT_NODE:
            newList.append(objList[n])
    return newList

def getCursor():
    sel = curSelection()
    result = None
    if sel == 0:
        messageBox(tr('error'), tr('noScore'))
        return result
    return sel

def changeDoc(score):
    
    global inst1, dar1, hals1,inst2, dar2, hals2,inst3, dar3, hals3,inst4, dar4, hals4,inst5, dar5, hals5,kopf1,kopf2,kopf3,kopf4,kopf5,stimmName,stimmabrev, doc

    sel = getCursor()
    staffpos = sel[0][1]
    system  = score.getElementsByTagName('system')
    for sy in range(system.length):
        staves = system[sy].getElementsByTagName('staves')
        for sv in range(staves.length):
            staff = staves[sv].getElementsByTagName('staff')
            staffname = staff[staffpos].getAttribute('layout')
            break
        break
      
    for staffLayout in score.getElementsByTagName('staffLayout'):          # Layout erweitern
        if staffLayout.getAttribute('description') == staffname:
            clone = staffLayout.cloneNode(True)
            sound = staffLayout.getElementsByTagName('sound')
            if sound[0].hasAttribute('transpose'):
                trans = sound[0].getAttribute('transpose')
            else:
                trans = '0'
            clone.setAttribute('description', staffname)
            staffLayout.parentNode.insertBefore(clone, staffLayout)
            staffLayout.setAttribute('description', stimmName)
            instrument = staffLayout.getElementsByTagName('instrument')
            TNname = instrument[0].getElementsByTagName('name')           # Stimmenbeschreibung eintragen
            TNname[0].appendChild(doc.createTextNode(stimmName))
            TNname = instrument[0].getElementsByTagName('abbrev')
            TNname[0].appendChild(doc.createTextNode(stimmabrev))                     
            instrument[0].setAttribute('name',stimmName)
            instrument[0].setAttribute('abbrev',stimmabrev)            

    for sys in system:                                                     # und Zeile clonen        
        for staff in sys.getElementsByTagName('staff'):
            if staff.getAttribute('layout') == staffname :
                clone = staff.cloneNode(True)
                staff.parentNode.insertBefore(clone, staff) 
                staff.setAttribute('layout', stimmName) 

    for sy in range(system.length):                                        # neue Schlagzeugzeile leeren und 5 Stimmen einfgen
        staves = system[sy].getElementsByTagName('staves')
        for sv in range(staves.length):
            staff = staves[sv].getElementsByTagName('staff')
            for st in range(staff.length):
                actual_staffname = staff[st].getAttribute('layout')
                if actual_staffname == stimmName:
                    voices = staff[st].getElementsByTagName('voices')
                    for vs in range(voices.length):
                        voice = voices[vs].getElementsByTagName('voice')
                        for vo in range(voice.length):
                            noteObjs  = voice[vo].getElementsByTagName('noteObjects')[0]
                            objList   = getElementObjects(noteObjs.childNodes)
                            for no in range(objList.length):
                                obj = objList[no]
                                cleanStaff(obj)                            # Staff leeren

                            insertVoices(staff, voice[vo])                 # 2. Stimme einfgen
                            insertVoices(staff, voice[vo])                 # 3. Stimme einfgen
                            insertVoices(staff, voice[vo])                 # 4. Stimme einfgen
                            insertVoices(staff, voice[vo])                 # 5. Stimme einfgen
    
    staffSourceName = staffname
    staffTargetName = stimmName

    for sy in range(system.length): 
        staves = system[sy].getElementsByTagName('staves')
        
        for sv in range(staves.length):
            staff = staves[sv].getElementsByTagName('staff')
            for st in range(staff.length):
                actual_staffname = staff[st].getAttribute('layout') 
                if actual_staffname == staffSourceName:
                    staffSource = staff[st]
                if actual_staffname == staffTargetName:
                    staffTarget = staff[st]
    
        voices = staffSource.getElementsByTagName('voices')
        for vs in range(voices.length):
            voice = voices[vs].getElementsByTagName('voice')
            for vo in range(voice.length):
                noteObjs  = voice[vo].getElementsByTagName('noteObjects')[0]
                objList   = getElementObjects(noteObjs.childNodes)
                for no in range(objList.length):
                    obj = objList[no]
                    if obj.nodeName == 'timeSign':
                        copyTime(obj,staffTarget)
                    if obj.nodeName == 'barline':
                        copyBarLine(obj,staffTarget)  
                    if obj.nodeName == 'chord':
                       copyChord(obj,staffTarget,trans) 
                    if obj.nodeName == 'rest':
                        copyRest(obj,staffTarget)  
                               
def pqDialog():
    global inst1,dar1,hals1,inst2,dar2,hals2,inst3,dar3,hals3,inst4,dar4,hals4,inst5,dar5,hals5,kopf1,kopf2,kopf3,kopf4,kopf5,stimmName,stimmabrev

    instrument  = [tr('notUsed'),'Accoustic Bass Drum','Bass Drum 1','SideStick','Accoustic Snare','Hand Clap','Electric Snare','Low Floor Tom','Closed Hi-Hat',
                   'High Floor Tom','Pedal Hi-Hat','Low Tom','Open Hi-Hat','Low Mid Tom','High Mid Tom','Crash Cymbal 1','High Tom','Ride Cymbal 1',
                    'Chinese Cymbal','Ride Bell','Tambourine','Splash Cymbal','Cow Bell','Crash Cymbal 2','Vibra Slap','Ride Cymbal 2',
                    'High Bongo','Low Bongo','Mute High Conga','Open high Conga','Low Conga','High Timbal','Low Timbal', 'High Agogo','Low Agogo','Cabasa',
                    'Maracas','Short Whistle','Long Whistle','Short Guiro','Long Guiro','Claves','High Wood Block','Low Wood Block','Mute Cuica','Open Cuica',
                    'Mute Triangle','Open Triangle','Cabasa 2','Jingle Bell','Bell Tree','Castanet','Side Stick','Taiko Low']   
    darstellung = ['c1','d1','e1','f1','g1','a1','h1','c2','d2','e2','f2','g2','a2','h2','c3','d3','e3','f3','g3'] 
    halsrichtung= [tr('up'),tr('down')] 
    kopf        = [tr('normal'),tr('cross'),tr('triangle'),tr('diamond'),tr('square'),tr('crossRing')] 
    stimmenname = ['Drum Set',tr('percussion')+'1',tr('percussion')+'2']
    stimmenabr  = ['D.S.','Perc 1','Perc 2']
    lablInst1   = Label('Instrument 1 ',width=7)
    editInst1   = ComboBox(instrument,width=20,value=0)   
    lablDar     = Label(tr('image'),width=5)
    editDar1    = ComboBox(darstellung,width=5,value=0) 
    lablKopf    = Label(tr('head'),width=5)
    editKopf1   = ComboBox(kopf,width=12,value=0)    
    lablHals    = Label(tr('stemDir'),width=5)
    editHals1   = ComboBox(halsrichtung,width=11,value=0) 
    lablInst2   = Label('Instrument 2 ',width=7)
    editInst2   = ComboBox(instrument,width=20,value=0)   
    lablDar     = Label(tr('image'),width=5)
    editDar2    = ComboBox(darstellung,width=5,value=0) 
    lablKopf    = Label(tr('head'),width=5)
    editKopf2   = ComboBox(kopf,width=12,value=0)    
    lablHals    = Label(tr('stemDir'),width=5)
    editHals2   = ComboBox(halsrichtung,width=11,value=0)   
    lablInst3   = Label('Instrument 3 ',width=7)
    editInst3   = ComboBox(instrument,width=20,value=8)   
    lablDar     = Label(tr('image'),width=5)
    editDar3    = ComboBox(darstellung,width=5,value=10) 
    lablKopf    = Label(tr('head'),width=5)
    editKopf3   = ComboBox(kopf,width=12,value=1)    
    lablHals    = Label(tr('stemDir'),width=5)
    editHals3   = ComboBox(halsrichtung,width=11,value=0)  
    lablInst4   = Label('Instrument 4 ',width=7)
    editInst4   = ComboBox(instrument,width=20,value=4)   
    lablDar     = Label(tr('image'),width=5)
    editDar4    = ComboBox(darstellung,width=5,value=7)
    lablKopf    = Label(tr('head'),width=5)
    editKopf4   = ComboBox(kopf,width=12,value=0)    
    lablHals    = Label(tr('stemDir'),width=5)
    editHals4   = ComboBox(halsrichtung,width=11,value=0)    
    lablInst5   = Label('Instrument 5 ',width=7)
    editInst5   = ComboBox(instrument,width=20,value=1)   
    lablDar     = Label(tr('image'),width=5)
    editDar5    = ComboBox(darstellung,width=5,value=3)
    lablKopf    = Label(tr('head'),width=5)
    editKopf5   = ComboBox(kopf,width=12,value=0)    
    lablHals    = Label(tr('stemDir'),width=5)
    editHals5   = ComboBox(halsrichtung,width=11,value=1) 
    lablName    = Label(tr('voiceName'),width=5)
    editName    = ComboBox(stimmenname,width=15,value=0)     

    box1        = HBox([lablInst1,editInst1,lablDar,editDar1,lablHals,editHals1,lablKopf,editKopf1,' '],padding=4) 
    box2        = HBox([lablInst2,editInst2,lablDar,editDar2,lablHals,editHals2,lablKopf,editKopf2,' '],padding=4) 
    box3        = HBox([lablInst3,editInst3,lablDar,editDar3,lablHals,editHals3,lablKopf,editKopf3,' '],padding=4) 
    box4        = HBox([lablInst4,editInst4,lablDar,editDar4,lablHals,editHals4,lablKopf,editKopf4,' '],padding=4) 
    box5        = HBox([lablInst5,editInst5,lablDar,editDar5,lablHals,editHals5,lablKopf,editKopf5,' '],padding=4) 
    box6        = HBox([lablName,editName,' '],text=tr('drumStaffName'),padding=4)
    boxA        = VBox([box1,box2,box3,box4,box5],text=tr('selInstr'),padding=4) 
    box         = VBox([box6,boxA],padding=4) 
    
    dlg = Dialog('Midi Drums   ' + version, box)
    if dlg.run():
        inst1       = editInst1.value()
        dar1        = editDar1.value()
        hals1       = editHals1.value()
        kopf1       = editKopf1.value()
        inst2       = editInst2.value()
        dar2        = editDar2.value()
        hals2       = editHals2.value()
        kopf2       = editKopf2.value()
        inst3       = editInst3.value()
        dar3        = editDar3.value()
        hals3       = editHals3.value()
        kopf3       = editKopf3.value()
        inst4       = editInst4.value()
        dar4        = editDar4.value()
        hals4       = editHals4.value()
        kopf4       = editKopf4.value()
        inst5       = editInst5.value()
        dar5        = editDar5.value()
        hals5       = editHals5.value()   
        kopf5       = editKopf5.value()
        nameIx      = editName.value()
        stimmName   = stimmenname [nameIx]
        stimmabrev  = stimmenabr [nameIx]

        return True
    else:
        return False
            
# Hauptprogramm:

from caplib.capDOM import ScoreChange
import tempfile

class ScoreChange(ScoreChange):
    def changeScore(self, score):
        global doc
        doc = score.parentNode
        changeDoc(score)

if pqDialog():
    if activeScore():
        activeScore().registerUndo('Mididrums')
        tempInput = tempfile.mktemp('.capx')
        tempOutput = tempfile.mktemp('.capx')        
        activeScore().write(tempInput)

        ScoreChange(tempInput, tempOutput)
        
        activeScore().read(tempOutput)
        os.remove(tempInput)
        os.remove(tempOutput)
            
